"use strict";

var Task = function() {

	this.init = function() {
		// Initialize the variables to draw on the canvas
		this.mycanvas = document.querySelector("#mycanvas");
		this.cx = this.mycanvas.getContext("2d");
		
		// Task parameters
		this.nBlocks = 3;
		this.nTrials = 5;
		this.currentBlock = 0;
		this.currentTrial = 0;
		
		// Response handler flags
		this.waiting_for_task_start = 0;
		this.waiting_for_block_start = 0;
		
		this.startTask();
	};

	// Helper functions

	this.showText = function(text0, x, y, color, fontsize) {
		if (color == undefined) {color = "white";}
		if (fontsize == undefined) {fontsize = 32;}
		this.cx.fillStyle = color;
		this.cx.font = fontsize + "px Georgia";
		this.cx.textAlign = "center";
		this.cx.textBaseline = "middle";		
		let x_screen = (this.mycanvas.width / 2) * (1 + x);
		let y_screen = (this.mycanvas.height / 2) * (1 + y);
		this.cx.fillText(text0, x_screen, y_screen);
	}

	this.emptyCanvas = function() {
		this.cx.fillStyle = "black";
		this.cx.fillRect(0, 0, this.mycanvas.width, this.mycanvas.height);
	};
	
	// Task start
	
	this.startTask = function() {
		this.showText("Hello World!", 0, 0);
		this.showText("Press a key to start", 0, 0.5);
		this.waiting_for_task_start = 1;
	}
	
	// Trials
	
	this.trial_init = function() {
		this.trial_fixation();
	}

	this.trial_fixation = function() {
		this.emptyCanvas();
		this.showText("+", 0, 0);
		setTimeout(this.trial_stimulus.bind(this), 250);
	}

	this.trial_stimulus = function() {
		this.emptyCanvas();
		this.showText(":-) Trial " + this.currentTrial, 0, 0);
		setTimeout(this.trial_ITI.bind(this), 500);
	}

	this.trial_ITI = function() {
		this.emptyCanvas();
		setTimeout(this.trial_end.bind(this), 250);
	}
	
	this.trial_end = function() {
		this.currentTrial++;
		if (this.currentTrial >= this.nTrials) {
			this.block_end();
		} else {
			this.trial_init();
		}
	}

	// Blocks
	this.block_init = function() {
		this.currentTrial = 0;
		
		this.emptyCanvas();
		this.showText("Block " + (this.currentBlock + 1), 0, 0);
		this.showText("Press a key to start", 0, 0.5);
		
		this.waiting_for_block_start = 1;
	}

	this.block_end = function() {
		this.currentBlock++;
		if (this.currentBlock >= this.nBlocks) {
			this.task_end();
		} else {
			this.block_init();
		}	
	}
	
	// End of task
	this.task_end = function() {
		this.showText("End", 0, 0);
	}
	
	// Response handling
	this.respHandler = function(event) {
		event.preventDefault();
		if (this.waiting_for_task_start == 1) {
			this.waiting_for_task_start = 0;
			this.block_init();
		} else if (this.waiting_for_block_start == 1) {
			this.waiting_for_block_start = 0;
			this.trial_init();
		}
	};
	
	window.addEventListener("keydown", this.respHandler.bind(this));

}