"use strict";

var Task = function() {

	this.init = function() {
		// Initialize the variables to draw on the canvas
		this.mycanvas = document.querySelector("#mycanvas");
		this.cx = this.mycanvas.getContext("2d");
		
		// Task parameters
		this.nBlocks = 3;
		this.nTrials = 5;
		this.currentBlock = 0;
		this.currentTrial = 0;
		
		// Response handler flags
		this.waiting_for_task_start = 0;
		this.waiting_for_block_start = 0;
		this.waiting_for_trial_response = 0;
		
		// Timing variables
		this.currentTimeout = 0;
		this.time_stimulus = 0;
		this.time_response = 0;
		
		// Performance variables
		this.RT = 0;
		this.respKeys = ["F", "J"];
		this.respCodes = [];
		for (var i = 0; i < this.respKeys.length; i++) {
			this.respCodes.push(this.respKeys[i].charCodeAt(0));
		}
		this.pressedRespKeyIndex = -1;
		this.pressedRespKey = "";
		this.acc = 0;
		
		// Stimuli
		this.stimuli = [":-)", ":-("];
		this.selectedStimulus = 0;
		
		this.startTask();
	};

	// Helper functions

	this.showText = function(text0, x, y, color, fontsize) {
		if (color == undefined) {color = "white";}
		if (fontsize == undefined) {fontsize = 32;}
		this.cx.fillStyle = color;
		this.cx.font = fontsize + "px Georgia";
		this.cx.textAlign = "center";
		this.cx.textBaseline = "middle";		
		let x_screen = (this.mycanvas.width / 2) * (1 + x);
		let y_screen = (this.mycanvas.height / 2) * (1 + y);
		this.cx.fillText(text0, x_screen, y_screen);
	}

	this.emptyCanvas = function() {
		this.cx.fillStyle = "black";
		this.cx.fillRect(0, 0, this.mycanvas.width, this.mycanvas.height);
	};
	
	// Task start
	
	this.startTask = function() {
		this.showText("Smiley and Frownies", 0, 0);
		this.showText("Smiley: Press F", 0, 0.2);
		this.showText("Frownie: Press J", 0, 0.4);
		this.showText("Press a key to start", 0, 0.8);
		this.waiting_for_task_start = 1;
	}
	
	// Trials
	
	this.trial_init = function() {
		// Default values
		this.RT = 0;
		this.acc = 0;
		this.pressedResponseKeyIndex = -1;
		this.pressedRespKey = "None";
		// Randomize stimuli
		this.selectedStimulus = Math.floor(Math.random() * 2);
		// Start first trial
		this.trial_fixation();
	}

	this.trial_fixation = function() {
		this.emptyCanvas();
		this.showText("+", 0, 0);
		setTimeout(this.trial_stimulus.bind(this), 250);
	}

	this.trial_stimulus = function() {
		this.emptyCanvas();
		let stimulusToShow = this.stimuli[this.selectedStimulus];
		this.showText(stimulusToShow, 0, 0);
		this.currentTimeout = setTimeout(this.trial_feedback.bind(this), 1000);
		this.waiting_for_trial_response = 1;
		this.time_stimulus = Date.now();
	}

	this.trial_feedback = function() {
		this.emptyCanvas();
		if (this.pressedRespKeyIndex == this.selectedStimulus) {
			this.acc = 1;
		} else {
			this.acc = 0;
		}
		if (this.acc == 1) {
			this.showText("Correct!", 0, 0, "green");
		} else {
			this.showText("Incorrect!", 0, 0, "red");
		}
		setTimeout(this.trial_ITI.bind(this), 500);
	}

	this.trial_ITI = function() {
		setTimeout(this.trial_end.bind(this), 250);
	}
	
	this.trial_end = function() {
		this.currentTrial++;
		if (this.currentTrial >= this.nTrials) {
			this.block_end();
		} else {
			this.trial_init();
		}
	}

	// Blocks
	this.block_init = function() {
		this.currentTrial = 0;
		
		this.emptyCanvas();
		this.showText("Block " + (this.currentBlock + 1), 0, 0);
		this.showText("Press a key to start", 0, 0.5);
		
		this.waiting_for_block_start = 1;
	}

	this.block_end = function() {
		this.currentBlock++;
		if (this.currentBlock >= this.nBlocks) {
			this.task_end();
		} else {
			this.block_init();
		}	
	}
	
	// End of task
	this.task_end = function() {
		this.showText("End", 0, 0);
	}
	
	// Response handling
	this.respHandler = function(event) {
		this.time_response = Date.now();
		event.preventDefault();
		if (this.waiting_for_task_start == 1) {
			this.waiting_for_task_start = 0;
			this.block_init();
		} else if (this.waiting_for_block_start == 1) {
			this.waiting_for_block_start = 0;
			this.trial_init();
		} else if (this.waiting_for_trial_response == 1) {
			this.waiting_for_trial_response = 0;
			// Get RT
			this.RT = this.time_response - this.time_stimulus;
			// Get pressed response key, as an index in the respKey array
			this.pressedRespKey = "Illegal key";
			for (var i = 0; i < this.respCodes.length; i++) {
				if (event.keyCode == this.respCodes[i]) {
					this.pressedRespKeyIndex = i;
					this.pressedRespKey = this.respKeys[this.pressedRespKeyIndex];
				}
			}
			clearTimeout(this.currentTimeout);
			this.trial_feedback();
		}
	};
	
	window.addEventListener("keydown", this.respHandler.bind(this));
	
}

