"use strict";

var Task = function() {

	this.init = function(UserID) {
		
		this.UserID = UserID;
		
		// Initialize the variables to draw on the canvas
		this.mycanvas = document.querySelector("#mycanvas");
		this.cx = this.mycanvas.getContext("2d");
		
		// Task parameters
		this.nBlocks = 3;
		this.nTrials = 5;
		this.currentBlock = 0;
		this.currentTrial = 0;
		
		// Response handler flags
		this.waiting_for_task_start = 0;
		this.waiting_for_block_start = 0;
		this.waiting_for_trial_response = 0;
		
		// Timing variables
		this.currentTimeout = 0;
		this.time_stimulus = 0;
		this.time_response = 0;
		
		// Performance variables
		this.RT = 0;
		this.respKeys = ["F", "J"];
		this.respCodes = [];
		for (var i = 0; i < this.respKeys.length; i++) {
			this.respCodes.push(this.respKeys[i].charCodeAt(0));
		}
		this.pressedRespKeyIndex = -1;
		this.pressedRespKey = "";
		this.acc = 0;
		
		// Stimuli
		this.selectedStimCat = 0;
		this.selectedStimulusInStimCat = 0;
		this.nStimCat = 2;
		this.nStim = [4, 4];
		this.stimFilenames = [];
		this.stimuli = [];
		this.nStimLoaded = 0;		
		
		// Saving data
		this.BlockData = [];
		
		// Task starts after stimuli are loaded
		this.init_stimuli();
	};

	// Stimulus loading

	this.init_stimuli = function() {
		// Just get the filenames
		for (var stimCat = 0; stimCat < this.nStimCat; stimCat++) {
			var subarr = [];
			for (var i = 0; i < this.nStim[stimCat]; i++) {
				subarr.push((function(i) {
					var fn0;
					if (stimCat == 0) fn0 = "Smilies/stim (" + (i + 1) + ").jpg";
					else fn0 = "Frownies/stim (" + (i + 1) + ").jpg";
					return fn0;
				})(i));
			}
			this.stimFilenames.push(subarr);
		}
		this.createNextStim(0, 0, []);		
	}

	this.createNextStim = function(stimCat, stimIndexInCat, subarr) {
		if (stimIndexInCat < this.nStim[stimCat] && stimCat < this.nStimCat) {
			var img = document.createElement("img");
			img.src = this.stimFilenames[stimCat][stimIndexInCat];
			img.width = 200;
			img.height = 200;
			subarr.push(img);
			stimIndexInCat++;
			img.addEventListener("load", this.loaded_a_stim.bind(this));
			this.createNextStim(stimCat, stimIndexInCat, subarr);
		} else {
			this.stimuli.push(subarr);
			stimCat++;
			if (stimCat < this.nStimCat) {
				subarr = [];
				stimIndexInCat = 0;
				this.createNextStim(stimCat, stimIndexInCat, subarr);
			}
		}
	};
	
	this.loaded_a_stim = function() {
		this.nStimLoaded++;
		var nStimTotal = 0;
		for (var n = 0; n < this.nStimCat; n++) {
			for (var m = 0; m < this.nStim[n]; m++) {
				nStimTotal++;
			}
		}
		this.showText(this.nStimLoaded + " loaded of " + nStimTotal);
		if (this.nStimLoaded >= nStimTotal) {
			this.startTask();
		}
	};
	
	// Helper functions

	this.showText = function(text0, x, y, color, fontsize) {
		if (color == undefined) {color = "white";}
		if (fontsize == undefined) {fontsize = 32;}
		this.cx.fillStyle = color;
		this.cx.font = fontsize + "px Georgia";
		this.cx.textAlign = "center";
		this.cx.textBaseline = "middle";		
		let x_screen = (this.mycanvas.width / 2) * (1 + x);
		let y_screen = (this.mycanvas.height / 2) * (1 + y);
		this.cx.fillText(text0, x_screen, y_screen);
	}

	this.showPicture = function(img, x, y, picsize) {
		var w = this.mycanvas.width * picsize;
		var h = this.mycanvas.height * picsize;
		var l = (this.mycanvas.width / 2) * (1 + x) - w/2;
		var t = (this.mycanvas.height / 2) * (1 + y) - h/2;
		this.cx.drawImage(img, l, t, w, h);
	}

	this.emptyCanvas = function() {
		this.cx.fillStyle = "black";
		this.cx.fillRect(0, 0, this.mycanvas.width, this.mycanvas.height);
	};
	
	// Task start
	
	this.startTask = function() {
		this.showText("Smiley and Frownies", 0, 0);
		this.showText("Smiley: Press F", 0, 0.2);
		this.showText("Frownie: Press J", 0, 0.4);
		this.showText("Press a key to start", 0, 0.8);
		this.waiting_for_task_start = 1;
	}
	
	// Trials
	
	this.trial_init = function() {
		// Default values
		this.RT = 0;
		this.acc = 0;
		this.pressedResponseKeyIndex = -1;
		this.pressedRespKey = "None";
		// Randomize stimuli
		this.selectedStimCat = Math.floor(Math.random() * 2);
		this.selectedStimulusInStimCat = Math.floor(Math.random() * this.nStim[this.selectedStimCat]);
		// Start first trial
		this.trial_fixation();
	}

	this.trial_fixation = function() {
		this.emptyCanvas();
		this.showText("+", 0, 0);
		setTimeout(this.trial_stimulus.bind(this), 250);
	}

	this.trial_stimulus = function() {
		this.emptyCanvas();
		let imgToShow = this.stimuli[this.selectedStimCat][this.selectedStimulusInStimCat];
		this.showPicture(imgToShow, 0, 0, 0.3);
		this.currentTimeout = setTimeout(this.trial_feedback.bind(this), 1000);
		this.waiting_for_trial_response = 1;
		this.time_stimulus = Date.now();
	}

	this.trial_feedback = function() {
		this.emptyCanvas();
		if (this.pressedRespKeyIndex == this.selectedStimCat) {
			this.acc = 1;
		} else {
			this.acc = 0;
		}
		if (this.acc == 1) {
			this.showText("Correct!", 0, 0, "green");
		} else {
			this.showText("Incorrect!", 0, 0, "red");
		}
		setTimeout(this.trial_ITI.bind(this), 500);
	}

	this.trial_ITI = function() {
		setTimeout(this.trial_end.bind(this), 250);
	}
	
	this.trial_end = function() {
		// Store trial data as a new sub-array in BlockData
		this.BlockData.push({currentBlock: this.currentBlock, currentTrial: this.currentTrial, stimCat: this.selectedStimCat, stimIndex:this.selectedStimulusInStimCat, respIndex: this.pressedRespKeyIndex, RT: this.RT, acc:this.acc});
		this.currentTrial++;
		if (this.currentTrial >= this.nTrials) {
			this.block_end();
		} else {
			this.trial_init();
		}
	}

	// Blocks
	this.block_init = function() {
		this.currentTrial = 0;
		
		this.BlockData.splice(0, this.BlockData.length)
	
		this.emptyCanvas();
		this.showText("Block " + (this.currentBlock + 1), 0, 0);
		this.showText("Press a key to start", 0, 0.5);
		
		this.waiting_for_block_start = 1;
	}

	this.block_end = function() {
		this.saveBlockData();
		this.currentBlock++;
		if (this.currentBlock >= this.nBlocks) {
			this.task_end();
		} else {
			this.block_init();
		}	
	}
	
	// Save data
	this.saveBlockData = function() {
		this.hr = new XMLHttpRequest();
		let dataString = "";
		for (var iTrial = 0; iTrial < this.BlockData.length; iTrial++) {
			let trialData = this.BlockData[iTrial];
			dataString = dataString + JSON.stringify(trialData) + "\n";
		}
		let fileName = "MyTask_" + this.UserID + ".txt";
		var vars = "fileName="+fileName+"&dataString="+dataString; 
		this.hr.open("POST", "savedata.php", true);
		this.hr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		this.hr.send(vars);
	}
	
	// End of task
	this.task_end = function() {
		this.showText("End", 0, 0);
	}
	
	// Response handling
	this.respHandler = function(event) {
		this.time_response = Date.now();
		event.preventDefault();
		if (this.waiting_for_task_start == 1) {
			this.waiting_for_task_start = 0;
			this.block_init();
		} else if (this.waiting_for_block_start == 1) {
			this.waiting_for_block_start = 0;
			this.trial_init();
		} else if (this.waiting_for_trial_response == 1) {
			this.waiting_for_trial_response = 0;
			// Get RT
			this.RT = this.time_response - this.time_stimulus;
			// Get pressed response key, as an index in the respKey array
			this.pressedRespKey = "Illegal key";
			for (var i = 0; i < this.respCodes.length; i++) {
				if (event.keyCode == this.respCodes[i]) {
					this.pressedRespKeyIndex = i;
					this.pressedRespKey = this.respKeys[this.pressedRespKeyIndex];
				}
			}
			clearTimeout(this.currentTimeout);
			this.trial_feedback();			
		}
	};
	
	window.addEventListener("keydown", this.respHandler.bind(this));
	
}

